@@ -14,18 +14,26 @@ G_id=as.numeric((levels(factor(group))))
 G=length(G_id)
 
 ########### Summary ###########
-n.event=n.death=n.censor=Z.event=Z.death=NULL
-for(i in G_id){
-  Gi=c(group==i)
-  n.event=c(n.event,sum(d1[Gi]))
-  n.death=c(n.death,sum(d2[Gi]))
-  n.censor=c(n.censor,sum(1-d2[Gi]))
-}
-count=cbind(G_id,N=table(group),n.event,n.death,n.censor)  
+n.event=tapply(d1, group, FUN=sum)
+n.death=tapply(d2, group, FUN=sum)
+n.censor=tapply(1-d2, group, FUN=sum)
+count=cbind(table(group),n.event,n.death,n.censor)  
+colnames(count)=c("No. of samples","No. of events","No. of deaths","No. of censors")
 
 xi1=min( T1 )
 xi3=max( T2 )
-######## Penalization term #########
+
+######## Choose smoothing parameters K1 and K2 ##########
+res1=splineCox.reg(t.event,event,Z1,xi1=xi1,xi3=xi3,
+                   kappa_grid=kappa_grid,LCV_plot=TRUE)
+res2=splineCox.reg(t.death,death,Z2,xi1=xi1,xi3=xi3,
+                   kappa_grid=kappa_grid,LCV_plot=TRUE)
+
+K1_est=res1$kappa
+LCV1_res=c(K1=K1_est,LCV1=res1$LCV)
+K2_est=res2$kappa
+LCV2_res=c(K2=K2_est,LCV2=res2$LCV)
+
 Omega=c(192,-132,24,12,0,
         -132,96,-24,-12,12,
         24,-24,24,-24,24,
@@ -33,93 +41,11 @@ Omega=c(192,-132,24,12,0,
         0,12,24,-132,192)
 Omega=matrix(Omega,5,5)/( (xi3-xi1)/2 )^5
 
-############ LCV for K1 ###############
-l1.func=function(phi){
-  beta1=phi[(5+1):(5+p1)]
-  g1=exp(  pmin(phi[1:5],500)  ) ## M-spline coefficients ##
-  l=-K1*t(g1)%*%Omega%*%g1
-  r1=as.vector( M.spline(T1,xi1=xi1,xi3=xi3)%*%g1 )
-  R1=as.vector( I.spline(T1,xi1=xi1,xi3=xi3)%*%g1 )
-  bZ1=as.numeric( Z1%*%beta1 )
-  l=l+sum( d1*(log(r1)+bZ1) )
-  l=l-sum(  pmin( exp(bZ1)*R1, exp(500) )  )
-  -l  
-}
-
-DF_upper=18+p1+p2
-
-L1=DF1=NULL
-
-for(k in 1:length(kappa_grid)){
-  K1=kappa_grid[k]
-  res1=nlm(l1.func,p=rep(0,5+p1),hessian=TRUE)
-  D1_PL=diag( c(1/exp(res1$estimate[1:5]),rep(1,p1)) )
-  H1_PL=-D1_PL%*%res1$hessian%*%D1_PL
-  H1=H1_PL
-  H1[1:5,1:5]=H1[1:5,1:5]+2*K1*Omega
-  K1=0
-  L1[k]=-l1.func(res1$estimate) 
-  if( is.na(det(H1_PL))|det(H1_PL)==0 ){DF1[k]=DF_upper}else{
-    DF1[k]=min( max( sum( diag(solve(H1_PL,tol=10^(-50))%*%H1) ), p1+2) ,DF_upper)
-  }
-}
-
-K1_est=kappa_grid[L1-DF1==max(L1-DF1)][1]
-LCV1_res=c(K1=K1_est,LCV1=max(L1-DF1))
-
-############ LCV for K2 ###############
-l2.func=function(phi){
-  beta2=phi[(5+1):(5+p2)]
-  g2=exp(  pmin(phi[1:5],500)  ) ## M-spline coefficients ##
-  l=-K2*t(g2)%*%Omega%*%g2
-  r2=as.vector( M.spline(T2,xi1=xi1,xi3=xi3)%*%g2 )
-  R2=as.vector( I.spline(T2,xi1=xi1,xi3=xi3)%*%g2 )
-  bZ2=as.numeric( Z2%*%beta2 )
-  l=l+sum( d2*(log(r2)+bZ2) )
-  l=l-sum(  pmin( exp(bZ2)*R2, exp(500) )  )
-  -l  
-}
-
-L2=DF2=NULL
-
-for(k in 1:length(kappa_grid)){
-  K2=kappa_grid[k]
-  res2=nlm(l2.func,p=rep(0,5+p2),hessian=TRUE)
-  D2_PL=diag( c(1/exp(res2$estimate[1:5]),rep(1,p2)) )
-  H2_PL=-D2_PL%*%res2$hessian%*%D2_PL
-  H2=H2_PL
-  H2[1:5,1:5]=H2[1:5,1:5]+2*K2*Omega
-  K2=0
-  L2[k]=-l2.func(res2$estimate) 
-  if( is.na(det(H2_PL))|det(H2_PL)==0 ){DF2[k]=DF_upper}else{
-   DF2[k]=min( max( sum( diag(solve(H2_PL,tol=10^(-50))%*%H2) ), p2+2), DF_upper)
-  }
-}
-
-K2_est=kappa_grid[L2-DF2==max(L2-DF2)][1]
-LCV2_res=c(K2=K2_est,LCV2=max(L2-DF2))
-
-########## Plotting LCV ##########
-if(LCV_plot==TRUE){
-  par(mfrow=c(1,3))
-  plot(kappa_grid,L1,xlab="K1",ylab="logL",type="b",lwd=3)
-  plot(kappa_grid,pmin(DF1,10+p1),xlab="K1",ylab="DF",,type="b",lwd=3)
-  plot(kappa_grid,L1-DF1,xlab="K1",ylab="LCV=logL-DF",type="b",lwd=3)
-  points(K1_est,max(L1-DF1),xlab="K1",col="red",pch=17,cex=2)
-
-  plot(kappa_grid,L2,xlab="K2",ylab="logL",type="b",lwd=3)
-  plot(kappa_grid,pmin(DF2,10+p2),xlab="K2",ylab="DF",type="b",lwd=3)
-  plot(kappa_grid,L2-DF2,xlab="K2",ylab="LCV=logL-DF",type="b",lwd=3)
-  points(K2_est,max(L2-DF2),col="red",pch=17,cex=2)
-}
-
 ############ Likelihood function ###############
 l.func=function(phi){
   
   g1=exp( pmax( pmin(phi[1:5],500), -500)  ) ## M-spline coefficients ## ver2.8
   g2=exp( pmax( pmin(phi[6:10],500),-500)  ) ## M-spline coefficients ##
-  #g1=exp(  pmin(phi[1:5],500)  ) ## M-spline coefficients ## ver2.7
-  #g2=exp(  pmin(phi[6:10],500)  ) ## M-spline coefficients ##
   eta=exp(phi[11])
   beta1=phi[(11+1):(11+p1)]
   beta2=phi[(11+p1+1):(11+p1+p2)]
@@ -167,14 +93,15 @@ ML=-res$minimum
 R_num=0
 repeat{
   if( (min( eigen(res$hessian)$values )>0)&(res$code==1) ){break}
+  if(R_num>=Randomize_num){break}
   R_num=R_num+1
-  if(R_num>Randomize_num){break}
   p0_Rand=runif(11+p1+p2,-1,1)
   res_Rand=nlm(l.func,p=p0_Rand,hessian=TRUE)
   ML_Rand=-res_Rand$minimum
   if(ML_Rand>ML){res=res_Rand}
 }
 H_PL=-res$hessian
+DF_upper=18+p1+p2
 
 temp=(det(H_PL)==0)|is.na(det(H_PL))
 if(temp){V=solve( -H_PL+diag(rep(0.0001,11+p1+p2)) ,tol=10^(-50))}else{V=solve(-H_PL,tol=10^(-50))}
